

### 1 - operations with vectors and matrices

# Scalar product between two vectors
a <- c(3, 2, -1)
b <- c(0, 2, -2)

a %*% b

3*0 + 2*2 + (-1) * (-2)

a <- rnorm(100)
b <- rnorm(100)

a %*% b

# Product between a matrix and a vector
A <- matrix(c(-2, 3, 1, 4), nrow = 2, ncol = 2, byrow= TRUE)
A
t(A).  # TRANSPOSE OF A MATRIX
b <- c(1, 3)
b

A %*% b

(-2) * 1 + 3 * 3
1 * 1 + 4 * 3


# Product between two matrices 
A <- matrix(c(-2, 3, 0, -1, 2, -1), nrow = 3, ncol = 2, byrow= TRUE)
B <- matrix(c(2, 1, 1, 4, 3, 0), nrow = 3, ncol = 2, byrow= TRUE)

A %*% t(B)   # 3 by 3
t(A) %*% B   # 2 by 2


### 2 - the simple linear model

# Load external data
data <- read.table("olympics.txt", header = TRUE)

# Define x and y
y <- data$time
x <- data$Year

# Check for missing data
is.na(x)
is.na(y)

# Remove missing data
x <- x[!is.na(y)]
y <- y[!is.na(y)]

# Estimate linear model with the function lm(), see results with summary()
mod <- lm(y ~ x)
summary(mod)

# Extract estimates "by hand"
var(x)
cov(x, y)

beta2_hat <- cov(x, y) / var(x)
beta1_hat <- mean(y) - beta2_hat * mean(x)

# See data and best fitting line
plot(x, y)
abline(a = beta1_hat, b = beta2_hat, col = "red")


# Extract estimates using matrix operations
n <- length(y)
X <- matrix(cbind(rep(1, n), x), nrow = n, ncol = 2, byrow = FALSE)
X

t(X) %*% X
crossprod(X)

t(X) %*% y
crossprod(X, y)

solve(crossprod(X))

beta_hat <- solve(crossprod(X)) %*% crossprod(X, y)

# compute fitted values
y_hat  <- predict(mod)
y_hat2 <- beta1_hat + beta2_hat * x
y_hat - y_hat2

# estimate variance of the error
sigma2_est <- crossprod(y - y_hat)/n 
sqrt(sigma2_est)


### 3 - diagnostics for the simple linear model

# Compute R-squared from correlation
cor(x, y)
cor(x, y)^2

# Compute R-squared from residual sum of squares
error <- y - y_hat 
sum(error, na.rm = T)
sum(error^2)

RSS <- sum(error^2)
RSS

TSS <- sum((y - mean(y))^2)
TSS

R2 <- 1 - RSS/TSS
R2

# Compute R-squared from explained sum of squares
ESS <- sum((y_hat - mean(y))^2)
ESS/TSS

# See confidence intervals (not reported by summary())
confint(mod)

# See residual plots

plot(mod)


### 4 - the multiple linear model

# Load internal data
data <- LifeCycleSavings

# Estimate linear model with the function lm(), see results with summary() and fitted()
mod2 <- lm(sr ~ pop15 + pop75 + dpi + ddpi, data = data)
summary(mod2)
y_hat <- fitted(mod2)

# NEW: compute F-statistic (RSS and TSS are required)

RSS <-  crossprod(y - y_hat)
RSS

TSS <- sum((data$sr - mean(data$sr))^2)
TSS

R2 <- 1 - RSS/TSS
R2

n <- nrow(data) 
p <- 4
Fstat <- ((TSS - RSS) / p) / (RSS / (n-p-1))
Fstat

# Check multicollinearity
cor(data[,-1])

# re-fit model with a subset of the variables
mod3 <- lm(sr ~ pop15 + ddpi, data = data)
summary(mod3)
plot(mod3)

